@@ -2,8 +2,8 @@
 # Coldcard Electrum plugin main code.
 #
 #
-import os, time, io
-import traceback
+import os
+import time
 from typing import TYPE_CHECKING, Optional
 import struct
 
@@ -14,13 +14,15 @@ from electrum.plugin import Device, hook, runs_in_hwd_thread
 from electrum.keystore import Hardware_KeyStore, KeyStoreWithMPK
 from electrum.transaction import PartialTransaction
 from electrum.wallet import Standard_Wallet, Multisig_Wallet, Abstract_Wallet
-from electrum.util import bfh, bh2u, versiontuple, UserFacingException
-from electrum.base_wizard import ScriptTypeNotSupported
+from electrum.util import bfh, versiontuple, UserFacingException
 from electrum.logging import get_logger
 
 from ..hw_wallet import HW_PluginBase, HardwareClientBase
 from ..hw_wallet.plugin import LibraryFoundButUnusable, only_hook_if_libraries_available
 
+if TYPE_CHECKING:
+    from electrum.plugin import DeviceInfo
+    from electrum.wizard import NewWalletWizard
 
 _logger = get_logger(__name__)
 
@@ -57,7 +59,6 @@ CKCC_SIMULATED_PID = CKCC_PID ^ 0x55aa
 
 
 class CKCCClient(HardwareClientBase):
-
     def __init__(self, plugin, handler, dev_path, *, is_simulator=False):
         HardwareClientBase.__init__(self, plugin=plugin)
         self.device = plugin.device
@@ -78,20 +79,28 @@ class CKCCClient(HardwareClientBase):
         # NOTE: MiTM test is delayed until we have a hint as to what XPUB we
         # should expect. It's also kinda slow.
 
+    def device_model_name(self) -> Optional[str]:
+        return 'Coldcard'
+
+    def get_soft_device_id(self) -> Optional[str]:
+        try:
+            super().get_soft_device_id()
+        except CCProtoError:
+            return None
+
     def __repr__(self):
         return '<CKCCClient: xfp=%s label=%r>' % (xfp2str(self.dev.master_fingerprint),
                                                         self.label())
 
     @runs_in_hwd_thread
-    def verify_connection(self, expected_xfp: int, expected_xpub=None):
+    def verify_connection(self, expected_xfp: int, expected_xpub: str):
         ex = (expected_xfp, expected_xpub)
 
         if self._expected_device == ex:
             # all is as expected
             return
 
-        if expected_xpub is None:
-            expected_xpub = self.dev.master_xpub
+        assert expected_xpub
 
         if ((self._expected_device is not None)
                 or (self.dev.master_fingerprint != expected_xfp)
@@ -110,9 +119,6 @@ class CKCCClient(HardwareClientBase):
 
         self._expected_device = ex
 
-        if not getattr(self, 'ckcc_xpub', None):
-            self.ckcc_xpub = expected_xpub
-
         _logger.info("Successfully verified against MiTM")
 
     def is_pairable(self):
@@ -142,7 +148,7 @@ class CKCCClient(HardwareClientBase):
 
         return lab
 
-    def manipulate_keystore_dict_during_wizard_setup(self, d: dict):
+    def _get_ckcc_master_xpub_from_device(self):
         master_xpub = self.dev.master_xpub
         if master_xpub is not None:
             try:
@@ -152,7 +158,7 @@ class CKCCClient(HardwareClientBase):
                     _('Invalid xpub magic. Make sure your {} device is set to the correct chain.').format(self.device) + ' ' +
                     _('You might have to unplug and plug it in again.')
                 ) from None
-            d['ckcc_xpub'] = master_xpub
+            return master_xpub
 
     @runs_in_hwd_thread
     def has_usable_connection_with_device(self):
@@ -160,7 +166,7 @@ class CKCCClient(HardwareClientBase):
         try:
             self.ping_check()
             return True
-        except:
+        except Exception:
             return False
 
     @runs_in_hwd_thread
@@ -187,7 +193,7 @@ class CKCCClient(HardwareClientBase):
         try:
             echo = self.dev.send_recv(CCProtocolPacker.ping(req))
             assert echo == req
-        except:
+        except Exception:
             raise RuntimeError("Communication trouble with Coldcard")
 
     @runs_in_hwd_thread
@@ -241,7 +247,7 @@ class CKCCClient(HardwareClientBase):
         # get a file
         return self.dev.download_file(length, checksum, file_number=file_number)
 
-        
+
 
 class Coldcard_KeyStore(Hardware_KeyStore):
     hw_type = 'coldcard'
@@ -269,6 +275,12 @@ class Coldcard_KeyStore(Hardware_KeyStore):
         assert xfp is not None
         return xfp_int_from_xfp_bytes(bfh(xfp))
 
+    def opportunistically_fill_in_missing_info_from_device(self, client: 'CKCCClient'):
+        super().opportunistically_fill_in_missing_info_from_device(client)
+        if self.ckcc_xpub is None:
+            self.ckcc_xpub = client._get_ckcc_master_xpub_from_device()
+            self.is_requesting_to_be_rewritten_to_wallet_file = True
+
     def get_client(self, *args, **kwargs):
         # called when user tries to do something like view address, sign somthing.
         # - not called during probing/setup
@@ -311,7 +323,7 @@ class Coldcard_KeyStore(Hardware_KeyStore):
         except (UnicodeError, AssertionError):
             # there are other restrictions on message content,
             # but let the device enforce and report those
-            self.handler.show_error('Only short (%d max) ASCII messages can be signed.' 
+            self.handler.show_error('Only short (%d max) ASCII messages can be signed.'
                                             % MSG_SIGNING_MAX_LENGTH)
             return b''
 
@@ -382,7 +394,7 @@ class Coldcard_KeyStore(Hardware_KeyStore):
                         break
 
                 rlen, rsha = resp
-            
+
                 # download the resulting txn.
                 raw_resp = client.download_file(rlen, rsha)
 
@@ -518,22 +530,6 @@ class ColdcardPlugin(HW_PluginBase):
             self.logger.exception('late failure connecting to device?')
             return None
 
-    def setup_device(self, device_info, wizard, purpose):
-        device_id = device_info.device.id_
-        client = self.scan_and_create_client_for_device(device_id=device_id, wizard=wizard)
-        return client
-
-    def get_xpub(self, device_id, derivation, xtype, wizard):
-        # this seems to be part of the pairing process only, not during normal ops?
-        # base_wizard:on_hw_derivation
-        if xtype not in self.SUPPORTED_XTYPES:
-            raise ScriptTypeNotSupported(_('This type of script is not supported with {}.').format(self.device))
-        client = self.scan_and_create_client_for_device(device_id=device_id, wizard=wizard)
-        client.ping_check()
-
-        xpub = client.get_xpub(derivation, xtype)
-        return xpub
-
     @runs_in_hwd_thread
     def get_client(self, keystore, force_pair=True, *,
                    devices=None, allow_user_interaction=True) -> Optional['CKCCClient']:
@@ -612,6 +608,30 @@ class ColdcardPlugin(HW_PluginBase):
             keystore.handler.show_error(_('This function is only available for standard wallets when using {}.').format(self.device))
             return
 
+    def wizard_entry_for_device(self, device_info: 'DeviceInfo', *, new_wallet=True) -> str:
+        if new_wallet:
+            return 'coldcard_start' if device_info.initialized else 'coldcard_not_initialized'
+        else:
+            return 'coldcard_unlock'
+
+    # insert coldcard pages in new wallet wizard
+    def extend_wizard(self, wizard: 'NewWalletWizard'):
+        views = {
+            'coldcard_start': {
+                'next': 'coldcard_xpub',
+            },
+            'coldcard_xpub': {
+                'next': lambda d: wizard.wallet_password_view(d) if wizard.last_cosigner(d) else 'multisig_cosigner_keystore',
+                'accept': wizard.maybe_master_pubkey,
+                'last': lambda d: wizard.is_single_password() and wizard.last_cosigner(d)
+            },
+            'coldcard_not_initialized': {},
+            'coldcard_unlock': {
+                'last': True
+            },
+        }
+        wizard.navmap_merge(views)
+
 
 def xfp_int_from_xfp_bytes(fp_bytes: bytes) -> int:
     return int.from_bytes(fp_bytes, byteorder="little", signed=False)
